@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2015 Peter Selinger.
+/* Copyright (C) 2001-2017 Peter Selinger.
    This file is part of Potrace. It is free software and it is covered
    by the GNU General Public License. See the file COPYING for details. */
 
@@ -13,7 +13,7 @@
 /* Note that bitmaps are stored bottom to top, i.e., the first
    scanline is the bottom-most one */
 
-extern char *bm_read_error;
+extern const char *bm_read_error;
 
 int bm_read(FILE *f, double blacklevel, potrace_bitmap_t **bmp);
 void bm_writepbm(FILE *f, potrace_bitmap_t *bm);
